@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2016 by Christian Dietrich                              *
  *   stettberger@dokucode.de                                               *
- *   Copyright (C) 2017-2018 by Hans-Peter Deifel <hpd@hpdeifel.de>        *
+ *   Copyright (C) 2017-2023 by Hans-Peter Deifel <hpd@hpdeifel.de>        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -32,35 +32,61 @@
 #include <unistd.h>
 #include <pwd.h>
 #include <sys/stat.h>
-#include <errno.h>
-#include <string.h>
+#include <cerrno>
+#include <cstring>
 
 using namespace std;
 
-Cache::Cache(string cache_file)
+const char *CACHE_VERSION = "1";
+
+static std::ostream& operator<<(std::ostream& out, const CachePage& page) {
+	out << page.label << '\0';
+	out << page.text << '\0';
+	return out;
+}
+
+static std::istream& operator>>(std::istream& in, CachePage& page) {
+	std::getline(in, page.label, '\0');
+	std::getline(in, page.text, '\0');
+	return in;
+}
+
+Cache::Cache(string const &cache_file)
 	: cache_file(cache_file), valid(false) {
 	// Open the cache file
 	ifstream fd(cache_file);
-	if (!fd) return;
+	if (!fd) {
+		return;
+	}
 
 	unsigned char indicator;
 	fd >> indicator;
-	if (indicator != 'G') return;
-	for (std::string page; std::getline(fd, page, '\0'); ) {
+	if (indicator != 'C') {
+		return;
+	}
+	std::string version;
+	std::getline(fd, version, '\0');
+	if (version != CACHE_VERSION) {
+		return;
+	}
+
+	for (CachePage page; fd >> page; ) {
 		pages.push_back(page);
 	}
 	valid = true;
 }
 
-void Cache::set_page(unsigned pagenum, const string &text) {
+void Cache::set_page(unsigned pagenum, const CachePage& page) {
 	pages.resize(max(pagenum, (unsigned)pages.size()));
-	pages[pagenum-1] = text;
+	pages[pagenum-1] = page;
 }
 
-bool Cache::get_page(unsigned pagenum, string &text) {
-	if (!valid) return false;
+bool Cache::get_page(unsigned pagenum, CachePage& page) {
+	if (!valid) {
+		return false;
+	}
 	if (pagenum-1 < pages.size()) {
-		text = pages[pagenum-1];
+		page = pages[pagenum-1];
 		return true;
 	}
 	return false;
@@ -68,19 +94,22 @@ bool Cache::get_page(unsigned pagenum, string &text) {
 
 void Cache::dump() {
 	ofstream fd(cache_file);
-	if (!fd) return;
-	// The first byte of the cache file is an indicator byte, which
-	// can be written atomically. Initial it is \0, after all pages
-	// have been flushed, the indicator byte is written to 'G' (for
-	// good).
-	fd << '\0';
-	for (const string &page : pages) {
-		fd << page;
-		fd << '\0';
+	if (!fd) {
+		return;
 	}
+	// The first byte of the cache file is an indicator byte, which can be
+	// written atomically. Initial it is \0, after all pages have been
+	// flushed, the indicator byte is written to 'C' (complete cache).
+	fd << '\0';
+	fd << CACHE_VERSION << '\0';
+
+	for (const CachePage& page : pages) {
+		fd << page;
+	}
+
 	fd.flush();
 	fd.seekp(0, ios_base::beg);
-	fd << 'G';
+	fd << 'C';
 	fd.close();
 }
 
@@ -91,39 +120,54 @@ static int agesort(const struct dirent ** a, const struct dirent **b) {
 	std::string B = string(cache_directory) + "/" + (*b)->d_name;
 
 	struct stat bufa, bufb;
-	if (stat(A.c_str(), &bufa) != 0) return 0;
-	if (stat(B.c_str(), &bufb) != 0) return 0;
+	if (stat(A.c_str(), &bufa) != 0) {
+		return 0;
+	}
+	if (stat(B.c_str(), &bufb) != 0) {
+		return 0;
+	}
 
 	return bufb.st_mtime - bufa.st_mtime;
 }
 
 static int agefilter(const struct dirent * a) {
-	if (a->d_name[0] == '.') return false;
+	if (a->d_name[0] == '.') {
+		return 0;
+	}
 	std::string A = string(cache_directory) + "/" + a->d_name;
 	struct stat bufa;
-	if (stat(A.c_str(), &bufa) != 0) return false;
+	if (stat(A.c_str(), &bufa) != 0) {
+		return 0;
+	}
 
 	// Filter all files that are younger than one day
-	return (time(NULL) - bufa.st_mtime) > 24 * 60 * 60;
+	if ((time(nullptr) - bufa.st_mtime) > 24 * 60 * 60) {
+		return 1;
+	}
+
+	return 0;
 }
 
 void limit_cachesize(const char *cache, int entries) {
 	struct dirent **namelist;
 	cache_directory = cache;
 	int n = scandir(cache, &namelist, agefilter, agesort);
+
 	if (n < 0) {
 		return;
-	} else {
-		while (entries--, n--) {
-			// Skip the first N cache entries
-			if (entries >= 0) continue;
-
-			string path(cache + string("/") + namelist[n]->d_name);
-			unlink(path.c_str());
-			free(namelist[n]);
-		}
-		free(namelist);
 	}
+
+	while (entries--, n-- > 0) {
+		// Skip the first N cache entries
+		if (entries >= 0) {
+			continue;
+		}
+
+		string path(cache + string("/") + namelist[n]->d_name);
+		unlink(path.c_str());
+		free(namelist[n]);
+	}
+	free(namelist);
 }
 
 int find_cache_directory(std::string &dir)
@@ -131,14 +175,14 @@ int find_cache_directory(std::string &dir)
 	const char *cache_base = getenv("XDG_CACHE_HOME");
 
 	dir = "";
-	if (cache_base != NULL && cache_base[0] != '\0') {
+	if (cache_base != nullptr && cache_base[0] != '\0') {
 		dir += cache_base;
 	} else {
 		char *home = getenv("HOME");
-		if (home == NULL) {
+		if (home == nullptr) {
 			struct passwd *passwd;
 			passwd = getpwuid(getuid());
-			if (passwd != NULL) {
+			if (passwd != nullptr) {
 				home = passwd->pw_dir;
 			} else {
 				return -1;
@@ -148,7 +192,7 @@ int find_cache_directory(std::string &dir)
 		dir += "/.cache";
 	}
 	// according to xdg spec, all directories should be created as 0700.
-	if (mkdir(dir.c_str(), 0700) && errno != EEXIST) {
+	if (mkdir(dir.c_str(), 0700) != 0 && errno != EEXIST) {
 		char *msg = strerror(errno);
 		err() << "mkdir(" << dir << "): " << msg << endl;
 		return -1;
@@ -156,7 +200,7 @@ int find_cache_directory(std::string &dir)
 
 	dir += "/pdfgrep/";
 
-	if (mkdir(dir.c_str(), 0700) && errno != EEXIST) {
+	if (mkdir(dir.c_str(), 0700) != 0 && errno != EEXIST) {
 		char *msg = strerror(errno);
 		err() << "mkdir(" << dir << "): " << msg << endl;
 		return -1;
