@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2016 by Christian Dietrich                              *
  *   stettberger@dokucode.de                                               *
- *   Copyright (C) 2017-2018 by Hans-Peter Deifel <hpd@hpdeifel.de>        *
+ *   Copyright (C) 2017-2023 by Hans-Peter Deifel <hpd@hpdeifel.de>        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -25,15 +25,20 @@
 #include <vector>
 #include <string>
 
+struct CachePage {
+	std::string text;
+	std::string label;
+};
+
 class Cache {
-	std::vector<std::string> pages;
+	std::vector<CachePage> pages;
 	std::string cache_file;
 	bool valid;
 public:
-	explicit Cache(std::string cache_file);
+	explicit Cache(std::string const& cache_file);
 
-	bool get_page(unsigned pagenum, std::string &text);
-	void set_page(unsigned pagenum, const std::string &text);
+	bool get_page(unsigned pagenum, CachePage& text);
+	void set_page(unsigned pagenum, const CachePage& page);
 
 	void dump();
 };
