@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2011-2018 by Hans-Peter Deifel                          *
+ *   Copyright (C) 2011-2023 by Hans-Peter Deifel                          *
  *   hpd@hpdeifel.de                                                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -33,11 +33,13 @@ void exclude_add(ExcludeList &list, const char *name)
 
 bool is_excluded(const ExcludeList &list, const char *name)
 {
-	ExcludeList::const_iterator iter = list.begin();
+	auto iter = list.begin();
 
+	// TODO Use range loop
 	for (; iter != list.end(); ++iter) {
-		if (!fnmatch(*iter, name, FNM_PATHNAME | FNM_PERIOD))
+		if (fnmatch(*iter, name, FNM_PATHNAME | FNM_PERIOD) == 0) {
 			return true;
+		}
 
 	}
 
